package cn.iocoder.yudao.module.member.service.ScoreRuleShareRecord;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.dal.dataobject.ScoreRuleShareRecord.ScoreRuleShareRecordDO;
import cn.iocoder.yudao.module.member.dal.mysql.ScoreRuleShareRecord.ScoreRuleShareRecordMapper;
import cn.iocoder.yudao.module.member.util.ScoreRuleGenCodeUtils;
import cn.iocoder.yudao.module.member.vo.ScoreRuleShareRecord.*;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;

/**
 * 分享记录 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class ScoreRuleShareRecordServiceImpl extends AbstractService<ScoreRuleShareRecordMapper, ScoreRuleShareRecordDO> implements ScoreRuleShareRecordService {

    @Resource
    private ScoreRuleShareRecordMapper scoreRuleShareRecordMapper;
    @Resource
    private ScoreRuleGenCodeUtils scoreRuleGenCodeUtils;

    @Override
    public Long createRuleShareRecord(ScoreRuleShareRecordCreateReqVO createReqVO) {
        // 校验是否已存在分享记录
        Long count = scoreRuleShareRecordMapper.selectCount(new LambdaQueryWrapperX<ScoreRuleShareRecordDO>()
                .eq(ScoreRuleShareRecordDO::getRuleId, createReqVO.getRuleId())
                .eq(ScoreRuleShareRecordDO::getMemberId, createReqVO.getMemberId()));
        if (count > 0) {
            return null;
        }
        ScoreRuleShareRecordDO scoreRuleShareRecordDO = new ScoreRuleShareRecordDO();
        scoreRuleShareRecordDO.setMemberId(createReqVO.getMemberId());
        scoreRuleShareRecordDO.setRuleId(createReqVO.getRuleId());
        scoreRuleShareRecordDO.setCode(scoreRuleGenCodeUtils.generateShareRecordCode());
        // 插入
        scoreRuleShareRecordMapper.insert(scoreRuleShareRecordDO);
        // 返回
        return scoreRuleShareRecordDO.getId();
    }

    @Override
    public PageResult<ScoreRuleShareRecordDetailVO> getRuleShareRecordDetail(ScoreRuleShareRecordDetailReqVO reqVO) {
        int size = reqVO.getRows();
        int start = (reqVO.getPage() - 1) * size;
        List<ScoreRuleShareRecordDetailVO> list = scoreRuleShareRecordMapper.getRuleShareRecordDetail(start, size, reqVO);
        if (list.isEmpty()) {
            return PageResult.empty();
        }
        list.stream().forEach(item -> {
            item.setTypeZh(DictFrameworkUtils.getDictDataFromCache("score_rule_type", item.getType().toString()).getLabel());
            item.setTypeEn(DictFrameworkUtils.getDictDataFromCache("score_rule_type", item.getType().toString()).getLabelEn());
        });
        int total = scoreRuleShareRecordMapper.countScoreRuleShareRecordDetail(reqVO);
        return new PageResult<>(list, total, size, reqVO.getPage(), (total + size - 1) / size);
    }

    @Override
    public PageResult<ScoreRuleShareRecordPageVO> getRuleShareRecordPage(ScoreRuleShareRecordQueryVO query) {
        int size = query.getRows();
        int start = (query.getPage() - 1) * size;
        List<ScoreRuleShareRecordPageVO> list = scoreRuleShareRecordMapper.getScoreRuleShareRecordList(start, size, query);
        if (list.isEmpty()) {
            return PageResult.empty();
        }
        list.stream().forEach(item -> {
            item.setTypeZh(DictFrameworkUtils.getDictDataFromCache("score_rule_type", item.getType().toString()).getLabel());
            item.setTypeEn(DictFrameworkUtils.getDictDataFromCache("score_rule_type", item.getType().toString()).getLabelEn());
        });
        int total = scoreRuleShareRecordMapper.countScoreRuleShareRecord(query);
        return new PageResult<>(list, total, size, query.getPage(), (total + size - 1) / size);
    }
}
